import React, {FunctionComponent, useCallback, useEffect, useState} from 'react';
import useServiceWorker from '../hooks/use-service-worker';
import PushSubscriptionDialog, {PushSubscriptionDialogProps} from './PushSubscriptionDialog';

export type PushSubscriptionProps = {
    debug?: boolean;
    publicKey: string;
    saveUrl: string;
    DialogProps: Omit<PushSubscriptionDialogProps, 'open'>;
}

const urlB64ToUint8Array = (base64String: string) => {
    const padding = '='.repeat((4 - base64String.length % 4) % 4);
    const base64 = (base64String + padding)
        .replace(/-/g, '+')
        .replace(/_/g, '/');

    const rawData = window.atob(base64);
    const outputArray = new Uint8Array(rawData.length);

    for (let i = 0; i < rawData.length; ++i) {
        outputArray[i] = rawData.charCodeAt(i);
    }
    return outputArray;
}

// ako ništa, za 24 sata
// ako ne, hvala za 45 dana

const PushSubscription: FunctionComponent<PushSubscriptionProps> = ({
    debug,
    DialogProps,
    publicKey,
    saveUrl
}) => {
    const registration = useServiceWorker();
    const [subscription, setSubscription] = useState<PushSubscription|null>();
    const [showDialog, setShowDialog] = useState(false);

    if (!('PushManager' in window)) {
        console.warn('No push support.');
        return null;
    }

    const subscribeUser = useCallback((registration?: ServiceWorkerRegistration|null) => {
        if (registration) {
            registration.pushManager.subscribe({
                userVisibleOnly: true,
                applicationServerKey: urlB64ToUint8Array(publicKey)
            })
                .then(subscription => {
                    setSubscription(subscription);
                })
                .catch(err => {
                    console.error('Error while subscribing to push.', err);
                });
        }
    }, [publicKey, registration]);

    const onDialogAcceptHandler = useCallback(() => {
        setShowDialog(false);
        subscribeUser(registration);
    }, [registration]);

    const onDialogDeclineHandler = useCallback(() => {
        localStorage.setItem('tiara_push_declined', '1');
        setShowDialog(false);
    }, []);

    useEffect(() => {
        if (registration) {
            {debug && console.log('@tiara/push: Getting the subscription...')}

            registration.pushManager.getSubscription()
                .then(subscription => {
                    if (subscription) {
                        const dec = new TextDecoder('utf-8');

                        if (dec.decode(urlB64ToUint8Array(publicKey)) !== dec.decode(subscription.options.applicationServerKey as ArrayBuffer)) {
                            subscription.unsubscribe().then(() => {
                                subscribeUser();
                            });
                        } else {
                            setSubscription(subscription);
                        }
                    } else if (debug) {
                        console.log('@tiara/push: No subscription found.');
                    }
                })
                .catch(err => {
                    console.error('@tiara/push: Error while getting push subscription.', err);
                });
        }
    }, [registration]);

    useEffect(() => {
        if (subscription) {
            fetch(saveUrl, {
                method: 'POST',
                headers: { 'Content-Type': 'application/json' },
                body: JSON.stringify(subscription.toJSON()),
            }).then();
        }
    }, [saveUrl, subscription]);

    if (Notification.permission === 'denied' || subscription) {
        return null;
    }

    const lastAsk = parseInt(localStorage.getItem('tiara_push_lastAsk') || '');
    const shouldAsk = !lastAsk ||
        (localStorage.getItem('tiara_push_declined') && (new Date()).getTime() - lastAsk > (86400000 * 45)) ||
        (!localStorage.getItem('tiara_push_declined') && (new Date()).getTime() - lastAsk > 86400000)
    ;

    if (shouldAsk) {
        setTimeout(() => {
            localStorage.setItem('tiara_push_lastAsk', (new Date()).getTime().toString());
            setShowDialog(true);
        }, 3000);
    }

    return (
        <PushSubscriptionDialog
            {...(DialogProps || {})}
            open={showDialog}
            onAccept={onDialogAcceptHandler}
            onDecline={onDialogDeclineHandler}
        />
    );
}

export default PushSubscription;
